from functools import partial


__all__ = ['decide', 'rules', 'NoMatchingRule']


def decide(rules_obj, *args, **kwargs):
    """
    Return first matching value from rules_obj by invoking each predicate
    within it with *args, **kwargs until a match is found. rules_obj
    must be an instance of rules.

    If no match is found, raise NoMatchingRule.

    If no arguments are passed, then return a partially applied function
    that can be later used to choose a value using a the given set of rules.

    """
    assert isinstance(rules_obj, rules)

    if not args and not kwargs:
        return partial(decide, rules_obj)

    for predicate, value in rules_obj:
        matches = predicate(*args, **kwargs)
        if matches:
            if isinstance(value, rules):
                return decide(value, *args, **kwargs)
            else:
                return value
    else:
        raise NoMatchingRule(args, kwargs)


class rules(object):
    """
    A list of predicates to be used in the decision process.

    Each rule is a 2-tuple (predicate, value). predicate is a function
    that returns True or False depending on whether the given arguments
    match some condition. value is the value that will be returned if the
    predicate matches. Rules can be nested by setting value to another
    rules object.
    """
    def __init__(self, *rules):
        self.__rules = rules

    def __iter__(self):
        for rule in self.__rules:
            yield rule


class NoMatchingRule(Exception):
    """
    No matching rule found for given value.
    """



import numbers
import collections


# catch-all predicate that always matches

otherwise = lambda *args, **kwargs: True


# types

instancechecker = lambda cls: lambda x: isinstance(x, cls)
subclasschecker = lambda cls: lambda x: issubclass(x, cls)

is_string = lambda x: isinstance(x, basestring)
is_number = lambda x: isinstance(x, numbers.Number)
is_callable = lambda x: isinstance(x, collections.Callable)
is_sequence = lambda x: isinstance(x, collections.Sequence)
is_mapping = lambda x: isinstance(x, collections.Mapping)


# identity checks

is_ = lambda y: lambda x: x is y
is_not = lambda y: lambda x: x is not y


# comparisons

eq = lambda y: lambda x: x == y
ne = lambda y: lambda x: x != y
gt = lambda y: lambda x: x > y
ge = lambda y: lambda x: x >= y
lt = lambda y: lambda x: x < y
le = lambda y: lambda x: x <= y


# collections

contains = lambda y: lambda x: y in x


# math

divisible_by = lambda *ys: lambda x: all(x % y == 0 for y in ys)



from setuptools import setup, find_packages

setup(
    name='judge',
    version='0.1.0',
    description='Choose values based on a tree of conditions',
    url='https://github.com/despawnerer/judge',
    author='Aleksei Voronov',
    author_email='despawn@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
)
